/*****************************************************************************
 *
 *  Copyright (C) 2007 Cdric Brgardis <cedric.bregardis AT free.fr> and
 *  Jean-Christian Hassler <jhassler AT free.fr>
 *
 *  This file is part of the Audiowerk2 ALSA driver
 *
 *  The Audiowerk2 ALSA driver is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2.
 *
 *  The Audiowerk2 ALSA driver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Audiowerk2 ALSA driver; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *****************************************************************************/

#include <sound/driver.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/slab.h>
#include <sound/core.h>
#include <sound/initval.h>
#include <sound/pcm.h>
#include <sound/pcm_params.h>

#include <linux/interrupt.h>
#include <asm/io.h>
#include "SAA7146.h"

#include <linux/delay.h>

#include "aw2-saa7146.h"

MODULE_LICENSE("GPL");
/*********************************
 * DEFINES
 ********************************/
#define PCI_VENDOR_ID_SAA7146         0x1131
#define PCI_DEVICE_ID_SAA7146         0x7146

//#define KERN_TRACE(args) (printf("DEBUG: "), printf args) 
//#define KERN_TRACE(args) (printk(KERN_ALERT, "DEBUG: %s %d", __FILE__, __LINE__), printk args)


/*********************************
 * TYPEDEFS
 ********************************/
  /* hardware definition */
static snd_pcm_hardware_t snd_aw2_playback_hw =
{
	.info = (SNDRV_PCM_INFO_MMAP |
                 SNDRV_PCM_INFO_INTERLEAVED |
                 SNDRV_PCM_INFO_BLOCK_TRANSFER |
                 SNDRV_PCM_INFO_MMAP_VALID),
	.formats =          SNDRV_PCM_FMTBIT_S16_LE,
	.rates =            SNDRV_PCM_RATE_44100,
	.rate_min =         44100,
	.rate_max =         44100,
	.channels_min =     2, /// @todo voir si on peux grer 1 pour faire du mono
	.channels_max =     4,
	.buffer_bytes_max = 32768,
	.period_bytes_min = 4096,
	.period_bytes_max = 32768,
	.periods_min =      1,
	.periods_max =      1024,
};

static snd_pcm_hardware_t snd_aw2_capture_hw =
{
	.info = (SNDRV_PCM_INFO_MMAP |
                 SNDRV_PCM_INFO_INTERLEAVED |
                 SNDRV_PCM_INFO_BLOCK_TRANSFER |
                 SNDRV_PCM_INFO_MMAP_VALID),
	.formats =          SNDRV_PCM_FMTBIT_S16_LE,
	.rates =            SNDRV_PCM_RATE_44100,
	.rate_min =         44100,
	.rate_max =         44100,
	.channels_min =     2, /// @todo voir si on peux grer 1 pour faire du mono
	.channels_max =     2,
	.buffer_bytes_max = 32768,
	.period_bytes_min = 4096,
	.period_bytes_max = 32768,
	.periods_min =      1,
	.periods_max =      1024,
};

typedef struct
{
    snd_pcm_t * pcm;
    unsigned int streamNumber;
    struct aw2_t_struct * chip;
} aw2_pcm_device_t;


typedef struct aw2_t_struct
{
    snd_aw2_saa7146_t saa7146;

    snd_card_t *card;
    
    aw2_pcm_device_t devicePlayback[NB_STREAM_PLAYBACK];
    aw2_pcm_device_t deviceCapture[NB_STREAM_CAPTURE];


//    spinlock_t reg_lock;
//    snd_pcm_substream_t * playback_substream[2];
} aw2_t;





/*********************************
 * FUNCTION DECLARATIONS
 ********************************/
static int __init alsa_card_aw2_init(void);
static void __exit alsa_card_aw2_exit(void);
static int snd_aw2_dev_free(snd_device_t *device);
static int __devinit snd_aw2_create(snd_card_t *card,
                                    struct pci_dev *pci,
                                    aw2_t **rchip);
static int __devinit snd_aw2_probe(struct pci_dev *pci,
                                   const struct pci_device_id *pci_id);
static void __devexit snd_aw2_remove(struct pci_dev *pci);
static int snd_aw2_pcm_playback_open(snd_pcm_substream_t *substream);
static int snd_aw2_pcm_playback_close(snd_pcm_substream_t *substream);
static int snd_aw2_pcm_capture_open(snd_pcm_substream_t *substream);
static int snd_aw2_pcm_capture_close(snd_pcm_substream_t *substream);
static int snd_aw2_pcm_hw_params(snd_pcm_substream_t *substream,
                                 snd_pcm_hw_params_t * hw_params);
static int snd_aw2_pcm_hw_free(snd_pcm_substream_t *substream);
static int snd_aw2_pcm_prepare_playback(snd_pcm_substream_t *substream);
static int snd_aw2_pcm_prepare_capture(snd_pcm_substream_t *substream);
static int snd_aw2_pcm_trigger_playback(snd_pcm_substream_t *substream, int cmd);
static int snd_aw2_pcm_trigger_capture(snd_pcm_substream_t *substream, int cmd);
static snd_pcm_uframes_t snd_aw2_pcm_pointer_playback(snd_pcm_substream_t *substream);
static snd_pcm_uframes_t snd_aw2_pcm_pointer_capture(snd_pcm_substream_t *substream);
static int __devinit snd_aw2_new_pcm(aw2_t *chip);


/*********************************
 * VARIABLES
 ********************************/
/* module parameters (see "Module Parameters") */
static int index[SNDRV_CARDS] = SNDRV_DEFAULT_IDX;
static char *id[SNDRV_CARDS] = SNDRV_DEFAULT_STR;
static int enable[SNDRV_CARDS] = SNDRV_DEFAULT_ENABLE_PNP;

static struct pci_device_id snd_aw2_ids[] = 
{
  { PCI_VENDOR_ID_SAA7146, PCI_DEVICE_ID_SAA7146,
    PCI_ANY_ID, PCI_ANY_ID, 0, 0, 0, },
  { 0, }
};

MODULE_DEVICE_TABLE(pci, snd_aw2_ids);

/* pci_driver definition */
static struct pci_driver driver =
{
  .name = "My Own Chip",
  .id_table = snd_aw2_ids,
  .probe = snd_aw2_probe,
  .remove = __devexit_p(snd_aw2_remove),
};

/* operators for playback PCM alsa interface */
static snd_pcm_ops_t snd_aw2_playback_ops = 
{
  .open =        snd_aw2_pcm_playback_open,
  .close =       snd_aw2_pcm_playback_close,
  .ioctl =       snd_pcm_lib_ioctl,
  .hw_params =   snd_aw2_pcm_hw_params,
  .hw_free =     snd_aw2_pcm_hw_free,
  .prepare =     snd_aw2_pcm_prepare_playback,
  .trigger =     snd_aw2_pcm_trigger_playback,
  .pointer =     snd_aw2_pcm_pointer_playback,
};

/* operators for capture PCM alsa interface */
static snd_pcm_ops_t snd_aw2_capture_ops = 
{
  .open =        snd_aw2_pcm_capture_open,
  .close =       snd_aw2_pcm_capture_close,
  .ioctl =       snd_pcm_lib_ioctl,
  .hw_params =   snd_aw2_pcm_hw_params,
  .hw_free =     snd_aw2_pcm_hw_free,
  .prepare =     snd_aw2_pcm_prepare_capture,
  .trigger =     snd_aw2_pcm_trigger_capture,
  .pointer =     snd_aw2_pcm_pointer_capture,
};

/*********************************
 * FUNCTION IMPLEMENTATIONS
 ********************************/


/* initialization of the module */
static int __init alsa_card_aw2_init(void)
{
  printk(KERN_ALERT "load aw2.o\n");
  return pci_register_driver(&driver);
}

/* clean up the module */
static void __exit alsa_card_aw2_exit(void)
{
  printk(KERN_ALERT "unload aw2.o\n");
  pci_unregister_driver(&driver);
}

module_init(alsa_card_aw2_init);
module_exit(alsa_card_aw2_exit);

/// @todo compile pas :
//EXPORT_NO_SYMBOLS; /* for old kernels only */


/* component-destructor
 * (see "Management of Cards and Components")
 */
static int snd_aw2_dev_free(snd_device_t *device)
{
  aw2_t *chip = device->device_data;
  return snd_aw2_saa7146_free(&chip->saa7146);
}

/* chip-specific constructor
 * (see "Management of Cards and Components")
 */
static int __devinit snd_aw2_create(snd_card_t *card,
                                       struct pci_dev *pci,
                                       aw2_t **rchip)
{
    aw2_t *chip;
    int err;
    static snd_device_ops_t ops =
        {
            .dev_free = snd_aw2_dev_free,
        };

    *rchip = NULL;

    /* initialize the PCI entry */
    if ((err = pci_enable_device(pci)) < 0)
        return err;
    pci_set_master(pci);

    /* check PCI availability (32bit DMA) */
    if (pci_set_dma_mask(pci, 0xffffffff) < 0 ||
        pci_set_consistent_dma_mask(pci, 0xffffffff) < 0)
    {
        printk(KERN_ERR "error to set 32bit mask DMA\n");
        pci_disable_device(pci);
        return -ENXIO;
    }
    chip = kcalloc(1, sizeof(*chip), GFP_KERNEL);
    if (chip == NULL)
    {
        pci_disable_device(pci);
        return -ENOMEM;
    }

    /* initialize the stuff */
    chip->card = card;
    chip->saa7146.pci = pci;
    chip->saa7146.irq = -1;

    /* (1) PCI resource allocation */
    if ((err = pci_request_regions(pci, "Audiowerk2")) < 0)
    {
        kfree(chip);
        return err;
    }
    chip->saa7146.iobase_phys = pci_resource_start(pci, 0);
    chip->saa7146.iobase_virt = ioremap_nocache(chip->saa7146.iobase_phys, pci_resource_len(pci, 0));
			          
    printk(KERN_ERR "grab irq %d\n", pci->irq);
    if (request_irq(pci->irq, snd_aw2_saa7146_interrupt,
                    SA_INTERRUPT|SA_SHIRQ, "Audiowerk2",
                    (void *)chip)) 
    {
        printk(KERN_ERR "cannot grab irq %d\n", pci->irq);
        snd_aw2_saa7146_free(&chip->saa7146);
        return -EBUSY;
    }
    chip->saa7146.irq = pci->irq;

    /* (2) initialization of the chip hardware */
    /// @todo
    snd_aw2_saa7146_setup(&chip->saa7146);

    if ((err = snd_device_new(card, SNDRV_DEV_LOWLEVEL, chip, &ops)) < 0) 
    {
        snd_aw2_saa7146_free(&chip->saa7146);
        return err;
    }

    snd_card_set_dev(card, &pci->dev);
    *rchip = chip;
    snd_aw2_saa7146_dump_registers(&chip->saa7146);
    return 0;
}


/* constructor -- see "Constructor" sub-section */
static int __devinit snd_aw2_probe(struct pci_dev *pci,
                                      const struct pci_device_id *pci_id)
{
    static int dev;
    snd_card_t *card;
    aw2_t *chip;
    int err;

    /* (1) Continue if device is not enabled, else inc dev */
    if (dev >= SNDRV_CARDS)
        return -ENODEV;
    if (!enable[dev]) 
    {
        dev++;
        return -ENOENT;
    }

    /* (2) Create card instance*/
    card = snd_card_new(index[dev], id[dev], THIS_MODULE, 0);
    if (card == NULL)
        return -ENOMEM;

    /* (3) Create main component*/
    if ((err = snd_aw2_create(card, pci, &chip)) < 0)
    {
        snd_card_free(card);
        return err;
    }

    /* initialize semaphore */
    init_MUTEX(&(chip->saa7146.sem));
    /* init spinlock */
    spin_lock_init(&chip->saa7146.reg_lock);
    /* initialize wait queue */
    init_waitqueue_head(&chip->saa7146.wait_iic);
    /* (4) Define driver ID and name string*/
    strcpy(card->driver, "aw2");
    strcpy(card->shortname, "Audiowerk2");

    sprintf(card->longname, "%s with SAA7146 at 0x%lx irq %i",
            card->shortname, chip->saa7146.port, chip->saa7146.irq);

    /* (5) Create other components */
    /// @todo 
    snd_aw2_new_pcm(chip);


    /* (6) Register card instance*/
    if ((err = snd_card_register(card)) < 0)
    {
        snd_card_free(card);
        return err;
    }

    /* (7) Set PCI driver data*/
    pci_set_drvdata(pci, card);

    dev++;
    return 0;
}

/* destructor -- see "Destructor" sub-section */
static void __devexit snd_aw2_remove(struct pci_dev *pci)
{
    snd_card_free(pci_get_drvdata(pci));
    pci_set_drvdata(pci, NULL);
}

/* open callback */
static int snd_aw2_pcm_playback_open(snd_pcm_substream_t *substream)
{
    //aw2_pcm_device_t * pcmDevice = snd_pcm_substream_chip(substream);
    snd_pcm_runtime_t *runtime = substream->runtime;

    printk(KERN_ALERT " playback_open \n");
    runtime->hw = snd_aw2_playback_hw;
    return 0;
}

/* close callback */
static int snd_aw2_pcm_playback_close(snd_pcm_substream_t *substream)
{
    ///@todo quelque chose  faire ?
    return 0;

}

static int snd_aw2_pcm_capture_open(snd_pcm_substream_t *substream)
{
    snd_pcm_runtime_t *runtime = substream->runtime;

    printk(KERN_ALERT " capture_open \n");
    runtime->hw = snd_aw2_capture_hw;
    return 0;
}

/* close callback */
static int snd_aw2_pcm_capture_close(snd_pcm_substream_t *substream)
{
    ///@todo quelque chose  faire ?
    return 0;
}

 /* hw_params callback */
static int snd_aw2_pcm_hw_params(snd_pcm_substream_t *substream,
                                    snd_pcm_hw_params_t * hw_params)
{
    printk(KERN_ALERT "Entree dans snd_aw2_pcm_hw_params  \n");
    return snd_pcm_lib_malloc_pages(substream, params_buffer_bytes(hw_params));
}

/* hw_free callback */
static int snd_aw2_pcm_hw_free(snd_pcm_substream_t *substream)
{
    return snd_pcm_lib_free_pages(substream);
}


/* prepare callback for playback */
static int snd_aw2_pcm_prepare_playback(snd_pcm_substream_t *substream)
{
    aw2_pcm_device_t * pcmDevice = snd_pcm_substream_chip(substream);
    aw2_t * chip = pcmDevice->chip;
    snd_pcm_runtime_t *runtime = substream->runtime;
    unsigned long periodSize, bufferSize;

    down(&chip->saa7146.sem);

    periodSize = snd_pcm_lib_period_bytes(substream);
    bufferSize = snd_pcm_lib_buffer_bytes(substream);

    printk(KERN_ALERT "snd_aw2_pcm_prepare: before configuration... (substream %d)\n",pcmDevice->streamNumber );
    snd_aw2_saa7146_pcm_init_playback(&chip->saa7146, pcmDevice->streamNumber , runtime->dma_addr, 
                                       periodSize, bufferSize);
    printk(KERN_ALERT "snd_aw2_pcm_prepare: After configuration\n");
    //printk("dma_addr:%x  periodSize:%x   bufferSize:%x\n",runtime->dma_addr, periodSize, bufferSize);

    /* Define Interrupt callback */
    snd_aw2_saa7146_define_it_playback_callback(pcmDevice->streamNumber , 
                                                (snd_aw2_saa7146_it_cb) snd_pcm_period_elapsed,
                                                (void*)substream);

    up(&chip->saa7146.sem);
	
    return 0;
}

/* prepare callback for capture */
static int snd_aw2_pcm_prepare_capture(snd_pcm_substream_t *substream)
{
    aw2_pcm_device_t * pcmDevice = snd_pcm_substream_chip(substream);
    aw2_t * chip = pcmDevice->chip;
    snd_pcm_runtime_t *runtime = substream->runtime;
    unsigned long periodSize, bufferSize;

    down(&chip->saa7146.sem);

    periodSize = snd_pcm_lib_period_bytes(substream);
    bufferSize = snd_pcm_lib_buffer_bytes(substream);

    printk(KERN_ALERT "snd_aw2_pcm_prepare: before configuration... (substream %d)\n",pcmDevice->streamNumber );
    snd_aw2_saa7146_pcm_init_capture(&chip->saa7146, pcmDevice->streamNumber , runtime->dma_addr, 
                                       periodSize, bufferSize);
    printk(KERN_ALERT "snd_aw2_pcm_prepare: After configuration\n");

    /* Define Interrupt callback */
    snd_aw2_saa7146_define_it_capture_callback(pcmDevice->streamNumber , 
                                               (snd_aw2_saa7146_it_cb) snd_pcm_period_elapsed,
                                               (void*)substream);

    up(&chip->saa7146.sem);
	
    return 0;
}

/* playback trigger callback */
static int snd_aw2_pcm_trigger_playback(snd_pcm_substream_t *substream, int cmd)
{
    int status = 0;
    aw2_pcm_device_t * pcmDevice = snd_pcm_substream_chip(substream);
    aw2_t * chip = pcmDevice->chip;
    spin_lock(&chip->saa7146.reg_lock);
    switch (cmd) {
        case SNDRV_PCM_TRIGGER_START:
            snd_aw2_saa7146_pcm_trigger_start_playback(&chip->saa7146, pcmDevice->streamNumber );
            break;
        case SNDRV_PCM_TRIGGER_STOP:
            snd_aw2_saa7146_pcm_trigger_stop_playback(&chip->saa7146, pcmDevice->streamNumber );
            break;
        default:
            status = - EINVAL;
    }
    spin_unlock(&chip->saa7146.reg_lock);
    return status;
}

/* capture trigger callback */
static int snd_aw2_pcm_trigger_capture(snd_pcm_substream_t *substream, int cmd)
{
    int status = 0;
    aw2_pcm_device_t * pcmDevice = snd_pcm_substream_chip(substream);
    aw2_t * chip = pcmDevice->chip;
    spin_lock(&chip->saa7146.reg_lock);
    switch (cmd) {
        case SNDRV_PCM_TRIGGER_START:
            snd_aw2_saa7146_pcm_trigger_start_capture(&chip->saa7146, pcmDevice->streamNumber );
            break;
        case SNDRV_PCM_TRIGGER_STOP:
            snd_aw2_saa7146_pcm_trigger_stop_capture(&chip->saa7146, pcmDevice->streamNumber );
            break;
        default:
            status = - EINVAL;
    }
    spin_unlock(&chip->saa7146.reg_lock);
    return status;
}

/* playback pointer callback */
static snd_pcm_uframes_t snd_aw2_pcm_pointer_playback(snd_pcm_substream_t *substream)
{
    aw2_pcm_device_t * pcmDevice = snd_pcm_substream_chip(substream);
    aw2_t * chip = pcmDevice->chip;
    unsigned int current_ptr;

    /* get the current hardware pointer */
    snd_pcm_runtime_t *runtime = substream->runtime;
    current_ptr = snd_aw2_saa7146_get_hw_pointer_playback(&chip->saa7146, pcmDevice->streamNumber ,
                                                          runtime->dma_area, runtime->buffer_size);
    
//    printk(KERN_ALERT "snd_aw2_pcm_pointer: current pointer:%d\n",(int)current_ptr);
    return bytes_to_frames(substream->runtime, current_ptr);

    /* This callback is called when the PCM middle layer inquires the current hardware position on the buffer.
       The position must be returned in frames (which was in bytes on ALSA 0.5.x), ranged from 0 to
       buffer_size - 1.
       This is called usually from the buffer-update routine in the pcm middle layer, which is invoked when
       snd_pcm_period_elapsed() is called in the interrupt routine. Then the pcm middle layer updates the
       position and calculates the available space, and wakes up the sleeping poll threads, etc.
       This callback is also atomic. */

}

/* capture pointer callback */
static snd_pcm_uframes_t snd_aw2_pcm_pointer_capture(snd_pcm_substream_t *substream)
{
    aw2_pcm_device_t * pcmDevice = snd_pcm_substream_chip(substream);
    aw2_t * chip = pcmDevice->chip;
    unsigned int current_ptr;

    /* get the current hardware pointer */
    snd_pcm_runtime_t *runtime = substream->runtime;
    current_ptr = snd_aw2_saa7146_get_hw_pointer_capture(&chip->saa7146, pcmDevice->streamNumber ,
                                                         runtime->dma_area, runtime->buffer_size);
    
//    printk(KERN_ALERT "snd_aw2_pcm_pointer: current pointer:%d\n",(int)current_ptr);
    return bytes_to_frames(substream->runtime, current_ptr);

    /* This callback is called when the PCM middle layer inquires the current hardware position on the buffer.
       The position must be returned in frames (which was in bytes on ALSA 0.5.x), ranged from 0 to
       buffer_size - 1.
       This is called usually from the buffer-update routine in the pcm middle layer, which is invoked when
       snd_pcm_period_elapsed() is called in the interrupt routine. Then the pcm middle layer updates the
       position and calculates the available space, and wakes up the sleeping poll threads, etc.
       This callback is also atomic. */

}

/* create a pcm device */
static int __devinit snd_aw2_new_pcm(aw2_t *chip)
{
    snd_pcm_t *pcmPlaybackAna;
    snd_pcm_t *pcmPlaybackNum;
    snd_pcm_t *pcmCapture;
    int err = 0;

    /* Create new Alsa PCM device */

    err = snd_pcm_new(chip->card, "Audiowerk2 analog playback", 0, 1, 0, &pcmPlaybackAna);
    if (err >= 0)
    {
        /* Creation ok */
        aw2_pcm_device_t * pcmDevice = &chip->devicePlayback[NUM_STREAM_PLAYBACK_ANA];

        /* Set PCM device name */
        strcpy(pcmPlaybackAna->name, "Analog playback");
        /* Associate private data to PCM device */
        pcmPlaybackAna->private_data = pcmDevice;       
        /* set operators of PCM device */
        snd_pcm_set_ops(pcmPlaybackAna, SNDRV_PCM_STREAM_PLAYBACK, &snd_aw2_playback_ops);
        snd_pcm_set_ops(pcmPlaybackAna, SNDRV_PCM_STREAM_CAPTURE, &snd_aw2_capture_ops);
        /* store PCM device */
        pcmDevice->pcm = pcmPlaybackAna;
        /* give base chip pointer to our internal pcm device structure */
        pcmDevice->chip = chip;
        /* Give stream number to PCM device */
        pcmDevice->streamNumber = NUM_STREAM_PLAYBACK_ANA;


        /* pre-allocation of buffers */
        /* Preallocate continuous pages. */
        // @todo voir si la taille convient pour les autres interface
        // (entre et sortie numrique)
        err = snd_pcm_lib_preallocate_pages_for_all(pcmPlaybackAna, SNDRV_DMA_TYPE_DEV,
                                                    snd_dma_pci_data(chip->saa7146.pci),
                                                    64*1024, 64*1024);
        if (err)
        {
            printk(KERN_ALERT "snd_aw2_new_pcm: snd_pcm_lib_preallocate_pages_for_all 1 error \n");
        }
    }

    err = snd_pcm_new(chip->card, "Audiowerk2 digital playback", 1, 1, 0, &pcmPlaybackNum);

    if (err >= 0)
    {
        /* Creation ok */
        aw2_pcm_device_t * pcmDevice = &chip->devicePlayback[NUM_STREAM_PLAYBACK_DIG];

        /* Set PCM device name */
        strcpy(pcmPlaybackNum->name, "Digital playback");
        /* Associate private data to PCM device */
        pcmPlaybackNum->private_data = pcmDevice;       
        /* set operators of PCM device */
        snd_pcm_set_ops(pcmPlaybackNum, SNDRV_PCM_STREAM_PLAYBACK, &snd_aw2_playback_ops);
        /* store PCM device */
        pcmDevice->pcm = pcmPlaybackNum;
        /* give base chip pointer to our internal pcm device structure */
        pcmDevice->chip = chip;
        /* Give stream number to PCM device */
        pcmDevice->streamNumber = NUM_STREAM_PLAYBACK_DIG;

        /* pre-allocation of buffers */
        /* Preallocate continuous pages. */
        // @todo voir si la taille convient pour les autres interface
        // (entre et sortie numrique)
        err = snd_pcm_lib_preallocate_pages_for_all(pcmPlaybackNum, SNDRV_DMA_TYPE_DEV,
                                                    snd_dma_pci_data(chip->saa7146.pci),
                                                    64*1024, 64*1024);
        if (err)
        {
            printk(KERN_ALERT "snd_aw2_new_pcm: snd_pcm_lib_preallocate_pages_for_all 1 error \n");
        }
    }

    err = snd_pcm_new(chip->card, "Audiowerk2 capture", 2, 0, 1, &pcmCapture);

    if (err >= 0)
    {
        /* Creation ok */
        aw2_pcm_device_t * pcmDevice = &chip->deviceCapture[NUM_STREAM_CAPTURE_ANA];

        /* Set PCM device name */
        strcpy(pcmCapture->name, "Capture");
        /* Associate private data to PCM device */
        pcmCapture->private_data = pcmDevice;       
        /* set operators of PCM device */
        snd_pcm_set_ops(pcmCapture, SNDRV_PCM_STREAM_CAPTURE, &snd_aw2_capture_ops);
        /* store PCM device */
        pcmDevice->pcm = pcmCapture;
        /* give base chip pointer to our internal pcm device structure */
        pcmDevice->chip = chip;
        /* Give stream number to PCM device */
        pcmDevice->streamNumber = NUM_STREAM_CAPTURE_ANA;

        /* pre-allocation of buffers */
        /* Preallocate continuous pages. */
        // @todo voir si la taille convient pour les autres interface
        // (entre et sortie numrique)
        err = snd_pcm_lib_preallocate_pages_for_all(pcmCapture, SNDRV_DMA_TYPE_DEV,
                                                    snd_dma_pci_data(chip->saa7146.pci),
                                                    64*1024, 64*1024);
        if (err)
        {
            printk(KERN_ALERT "snd_aw2_new_pcm: snd_pcm_lib_preallocate_pages_for_all 1 error \n");
        }
    }
    return 0;
}          
