/*****************************************************************************
 *
 *  Copyright (C) 2007 Cdric Brgardis <cedric.bregardis AT free.fr> and
 *  Jean-Christian Hassler <jhassler AT free.fr>
 *
 *  This file is part of the Audiowerk2 ALSA driver
 *
 *  The Audiowerk2 ALSA driver is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; version 2.
 *
 *  The Audiowerk2 ALSA driver is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with the Audiowerk2 ALSA driver; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *****************************************************************************/

#ifndef AW2_SAA7146
#define AW2_SAA7146

#ifdef AW2_SAA7146_M
#define PUBLIC
#else
#define PUBLIC extern
#endif


#define NB_STREAM_PLAYBACK 2
#define NB_STREAM_CAPTURE 1

#define NUM_STREAM_PLAYBACK_ANA 0
#define NUM_STREAM_PLAYBACK_DIG 1

#define NUM_STREAM_CAPTURE_ANA 0

typedef void (*snd_aw2_saa7146_it_cb)(void *);

typedef struct
{
    snd_aw2_saa7146_it_cb pItCallback;
    void * pCallbackParam;
} snd_aw2_saa7146_cb_param;

/* definition of the chip-specific record */
//typedef struct snd_aw2 snd_aw2_saa7146_t;

typedef struct
{
    struct pci_dev *pci;
    
    unsigned long iobase_phys;
    void __iomem *iobase_virt;
    unsigned long port;
    int irq;

    struct semaphore sem;
    spinlock_t reg_lock;
    int iic;
    wait_queue_head_t wait_iic;
} snd_aw2_saa7146_t;


PUBLIC void snd_aw2_saa7146_dump_registers(snd_aw2_saa7146_t  *chip);


PUBLIC int snd_aw2_saa7146_free(snd_aw2_saa7146_t *chip);
PUBLIC void snd_aw2_saa7146_setup(snd_aw2_saa7146_t  *chip);
PUBLIC void snd_aw2_saa7146_pcm_init_playback(snd_aw2_saa7146_t *chip,
                                              int stream_number, 
                                              unsigned long dma_addr,
                                              unsigned long period_size,
                                              unsigned long buffer_size);
PUBLIC void snd_aw2_saa7146_pcm_init_capture(snd_aw2_saa7146_t *chip,
                                             int stream_number, 
                                             unsigned long dma_addr,
                                             unsigned long period_size,
                                             unsigned long buffer_size);
PUBLIC void snd_aw2_saa7146_define_it_playback_callback(unsigned int stream_number,
                                                        snd_aw2_saa7146_it_cb pItCallback,
                                                        void * pCallbackParam);
PUBLIC void snd_aw2_saa7146_define_it_capture_callback(unsigned int stream_number,
                                                       snd_aw2_saa7146_it_cb pItCallback,
                                                       void * pCallbackParam);
PUBLIC void snd_aw2_saa7146_pcm_trigger_start_capture(snd_aw2_saa7146_t *chip, int stream_number);
PUBLIC void snd_aw2_saa7146_pcm_trigger_stop_capture(snd_aw2_saa7146_t *chip, int stream_number);

PUBLIC void snd_aw2_saa7146_pcm_trigger_start_playback(snd_aw2_saa7146_t *chip, int stream_number);
PUBLIC void snd_aw2_saa7146_pcm_trigger_stop_playback(snd_aw2_saa7146_t *chip, int stream_number);

PUBLIC irqreturn_t snd_aw2_saa7146_interrupt(int irq, void *dev_id, struct pt_regs *regs);
PUBLIC unsigned int snd_aw2_saa7146_get_hw_pointer_playback(snd_aw2_saa7146_t *chip, int stream_number,
                                                            unsigned char * startAddr, unsigned int bufferSize);
PUBLIC unsigned int snd_aw2_saa7146_get_hw_pointer_capture(snd_aw2_saa7146_t *chip, int stream_number,
                                                           unsigned char * startAddr, unsigned int bufferSize);
PUBLIC int snd_aw2_saa7146_set_sample_rate(snd_aw2_saa7146_t * chip, int srate);

#endif
